package PatchSystemInfoHandler;

import static org.junit.Assert.*;

import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;

import org.junit.Test;


public class JTest {

	/**
	 * 
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 */
	private static void init() throws InstantiationException, IllegalAccessException, ClassNotFoundException {
		
		//connect to database patch system info db
		PatchSystemInfoHandler.createConnection();
		
		/*
		PatchSystemInfoHandler.add(1, "1.1.1");
		PatchSystemInfoHandler.add(2, "1.1.2");
		PatchSystemInfoHandler.add(3, "1.1.3");
		PatchSystemInfoHandler.add(4, "1.2.1");
		PatchSystemInfoHandler.add(5, "1.2.2");
		PatchSystemInfoHandler.add(6, "1.2.3");
		PatchSystemInfoHandler.add(7, "2.1.1");
		PatchSystemInfoHandler.add(1, "2.1.2");
		PatchSystemInfoHandler.add(1, "2.1.3");
		
		try {
			String query = "INSERT INTO patchsystemattribute (keyname, value) "
					+ "VALUES ('ePagesTargetVersion', '2.1.3')";
			Statement stmt = PatchSystemInfoHandler.con.createStatement();
			stmt.executeQuery(query);	
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}*/
		
	}
	
	private static void cleanUp() throws SQLException {	
		//close connection to database patchsysteminfodb
		PatchSystemInfoHandler.closeConnection();
	}
	
	
	@Test
	public static void test() throws Exception {
	
		//test preparation
		init();
		
		System.out.println("-------------------------------start test-------------------------------");
		
		//PatchSystemInfoHandler.setStartTime();
		Time startTime = PatchSystemInfoHandler.getStartTime();
		System.out.println(String.format("Start: %s", startTime.toString()));
		
		//TODO testing return true
		assertFalse(PatchSystemInfoHandler.existsLanguage("English")); 
		System.out.println("existsLanguage passed");
		
		//TODO testing return true
		assertFalse(PatchSystemInfoHandler.isCartridgeFunctionCompleted("6.16.1", "Site", "DE_EPAGES::DHL", "patch"));
		System.out.println("isCartridgeFunctionCompleted passed");
		
		//TODO testing return true
		assertFalse(PatchSystemInfoHandler.isCartridgeFunctionStarted("6.16.1", "Site", "DE_EPAGES::DHL", "patch"));
		System.out.println("isCartridgeFunctionStarted passed");
		
		//TODO testing return true
		assertFalse(PatchSystemInfoHandler.isStoreCartridgeStarted("6.16.1", "Site", "DE_EPAGES::DHL"));
		System.out.println("isStoreCartridgeStarted passed");
		
		assertTrue(PatchSystemInfoHandler.validBasicAttribute("DataBaseSystem"));
		assertFalse(PatchSystemInfoHandler.validBasicAttribute("epagesdir"));
		System.out.println("validBasicAttribute passed");
		
		assertTrue(PatchSystemInfoHandler.validVersion("6.17.0"));
		assertFalse(PatchSystemInfoHandler.validVersion("6.18.4"));
		System.out.println("validVersion passed");
		
		assertTrue(PatchSystemInfoHandler.virtualBasicAttribute("ePagesCurrentVersion"));
		assertFalse(PatchSystemInfoHandler.virtualBasicAttribute("ePagesPastVersion"));
		System.out.println("virtualBasicAttribute passed");
		
		assertTrue(PatchSystemInfoHandler.existsBasicAttribute("DataBaseSystem"));
		assertFalse(PatchSystemInfoHandler.existsBasicAttribute("database"));
		System.out.println("existsBasicAttribute passed");
		
		assertTrue(PatchSystemInfoHandler.validHandler("API::Patch::PatchInstallHandler"));
		assertFalse(PatchSystemInfoHandler.validHandler("API:Patch:PatchInstallhandler"));
		System.out.println("validHandler passed");
		
		assertTrue(PatchSystemInfoHandler.existsPatchEvent("PatchStartTime"));
		assertFalse(PatchSystemInfoHandler.existsPatchEvent("PatchStarts"));
		System.out.println("existsPatchEvent passed");
		
		assertTrue(PatchSystemInfoHandler.validInstallerFunction("patch"));
		assertFalse(PatchSystemInfoHandler.validInstallerFunction("delete"));
		System.out.println("validInstallerFunction passed");
		
		assertTrue(PatchSystemInfoHandler.lockedVersionKey("PatchWebrootVersionFolder"));
		assertFalse(PatchSystemInfoHandler.lockedVersionKey("LanguagePackVersion"));
		System.out.println("lockedVersionKey passed");
		
		assertEquals("patch_6_17_5", PatchSystemInfoHandler.getPatchTag());
		System.out.println("getPatchTag passed");
		
		assertTrue(PatchSystemInfoHandler.isLastVersion("6.17.5"));
		assertFalse(PatchSystemInfoHandler.isLastVersion("6.17.4"));
		System.out.println("isLastVersion passed");
		
		assertTrue(PatchSystemInfoHandler.isFirstVersion("6.13.3"));
		assertFalse(PatchSystemInfoHandler.isFirstVersion("6.13.4"));
		System.out.println("isFirstVersion passed");
		
		
		//check return integer, String functions
		
		assertEquals("/root/patch_6_17_5", PatchSystemInfoHandler.getBasicAttribute("PatchDir"));
		assertFalse("6.17.5".equals(PatchSystemInfoHandler.getBasicAttribute("PatchDir")));
		System.out.println("getBasicAttribute passed");
		
		//getallVersions
		String[] expectedResult = {"6.13.3", "6.13.4", "6.14.0", "6.14.1", "6.14.2", "6.14.3", "6.14.4",
                "6.15.0", "6.15.1", "6.15.2", "6.15.3", "6.15.4", "6.16.0", "6.16.1","6.16.2", "6.16.3", 
                "6.17.0", "6.17.1", "6.17.2", "6.17.3", "6.17.4", "6.17.5"};	
		String[] result = PatchSystemInfoHandler.validVersions().toArray(new String[expectedResult.length]);	
		assertEquals(expectedResult.length, result.length);
		for(int i=0; i < expectedResult.length; i++) {
			assertEquals("mismatch at " + i, expectedResult[i], result[i]);
		}
		System.out.println("getallVersions passed");

		assertEquals("/srv/epages/eproot/Cartridges", PatchSystemInfoHandler.getVersionAttribute("6.16.3", "CartridgesVersionFolder"));
		assertFalse("getVersionAttribute false fail", "/srv/epages".equals(PatchSystemInfoHandler.getVersionAttribute("6.13.5", "CartridgeVersion")));
		System.out.println("getVersionAttribute passed");		
		
		//time test finished
		Time finishedTime = PatchSystemInfoHandler.getCurrentTime();
		System.out.println(String.format("Finished: %s", finishedTime.toString()));
		
		PatchSystemInfoHandler.collectSystemInfo();
		
		System.out.println("------------------------test finished successfull------------------------");
	}
	
	public static void main(String[] args) throws Exception {
		test();
		cleanUp();
	}

}
