/**
 * 
 */
package PatchSystemInfoHandler;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Nicole Stender
 * @version 0.1
 */
public class PatchSystemInfoHandler {
	
	// Database Connection vars
	static Connection con = null; //database connector
	private static String pwd = "epages";
	private static String user = "usr_patch";
	private static String dbURL = "jdbc:mysql://localhost/patchsysteminfodb";
	
	//SQL vars
	private static Statement stmt = null; 
	private static PreparedStatement prepState = null; 
	private static ResultSet result = null;
	private static String query = null;
	
	//init InstallerFunctionInfoHash
	private static List<String> installerFunction = Arrays.asList("patch", "patchDBStructure", "install", "patchCartridgeDir");
	private static Map<Integer, List<String>> installerFunctionInfoHash;
	static {
		installerFunctionInfoHash = new HashMap<Integer, List<String>>();
		installerFunctionInfoHash.put(1, installerFunction);
	}
	
	
	/**
	 * Description: Creates connection to database patchsysteminfodb
	 * @return: Connector to database | Connection
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 */
	public static Connection createConnection () throws InstantiationException, IllegalAccessException, ClassNotFoundException {
					
		try {
			Class.forName("com.mysql.jdbc.Driver");
			con = DriverManager.getConnection(dbURL, user, pwd);
			System.out.println("Connection to database 'PatchSystemInfoDB' created.");
		} catch(SQLException e) {
			//handle errors
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState: " + e.getSQLState());
			System.out.println("VendorError: " + e.getErrorCode());
		}
		return con;
	}
	
	/**
	 * Close connection to database
	 * @throws SQLException
	 */
	public static void closeConnection () throws SQLException {
		stmt.close();
		prepState.close();
		result.close();
		con.close();
		System.out.println("Connection to database 'PatchSystemInfoDB' closed.");
	}
	
	/**
	 * check if the patch handler package is already in the table 'patchhandler'
	 * @param pack | package of patch handler object | String
	 * @return true if already exists, false otherwise | boolean
	 * @throws SQLException
	 */
	private static boolean existsPatchHandlerInDB(String pack) throws SQLException {
		pack = getHandlerPackage(pack);
		try {
			query = "SELECT * FROM patchhandler WHERE package= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, pack);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			if(pack.equals(result.getString(2))) {
				return true;
			}else {
				return false;
			}
		}else {
			return false;
		}
	}
	
	/**
	 * fill the table 'patchgroundfunction'
	 * @throws SQLException
	 */
	private static void initializeDB_PatchGroundFunction() throws SQLException {
		
		//ground functions
		ArrayList<String> validGroundFunctions = new ArrayList<String>();
		validGroundFunctions.add("doPatch");
		validGroundFunctions.add("callSubHandlerWithVersionSources");
		validGroundFunctions.add("callSubHandlerWithVersionSourcesAsBackgroundProcess");
		validGroundFunctions.add("toExecute");
		validGroundFunctions.add("doPatch_checkPreCondition");
		validGroundFunctions.add("doPatch_basic");
		validGroundFunctions.add("doPatch_universal");
		validGroundFunctions.add("useNow");
		
		//add versionized functions to possible ground functions
		ArrayList<String> validVersion = validVersions();
		for(int i = 0; i < validVersion.size(); i++) {
			String validVersionUnderscored = validVersion.get(i);
			
			//get underscored version
			validVersionUnderscored.replace('.', '_');
			validGroundFunctions.add("doPatch_" + validVersionUnderscored);
		}
		
		//placeholder log level, default level -1 
		int LogLevel = -1;
		
		//foreach valid groundfunction check if it is in db
		for(int i = 0; i < validGroundFunctions.size(); i++) {
			if(existsPatchGroundFunctionInDB(validGroundFunctions.get(i))) {
				continue;
			} else {
				//change to switch(..) { } with JDK7, switch(String) {} not supported in JDK6				
				if(validGroundFunctions.get(i).equals("doPatch")) {
					LogLevel = 4;				
				} else if(validGroundFunctions.get(i).equals("callSubHandlerWithVersionSources") || 
						validGroundFunctions.get(i).equals("callSubHandlerWithVersionAsBackgroundProcess")) {
					LogLevel = 3;
				} else if(validGroundFunctions.get(i).equals("toExecute")) {
					LogLevel = 2;
				} else if(validGroundFunctions.get(i).equals("doPatch_checkPreCondition")) {
					LogLevel = 2;
				} else if(validGroundFunctions.get(i).equals("doPatch_basic")) {
					LogLevel = 1;
				} else if(validGroundFunctions.get(i).equals("useNow")) {
					LogLevel = 0;
				} else if(validGroundFunctions.get(i).contains("doPatch_")) {
					LogLevel = 0;
				} else {
					System.out.println(String.format("missing log level for patch ground function %s", validGroundFunctions.get(i)));
				}
				
				try {
					query = "INSERT INTO patchgroundfunction (functionname, loglevel) VALUES (?, ?)";
					prepState = con.prepareStatement(query);
					prepState.setString(1, validGroundFunctions.get(i));
					prepState.setInt(2, LogLevel);
					result = prepState.executeQuery();
				} catch(SQLException e) {
					//handle errors
				    System.out.println("SQLException: " + e.getMessage());
				    System.out.println("SQLState: " + e.getSQLState());
				    System.out.println("VendorError: " + e.getErrorCode());
				}
			}
		}
	}
	
	/**
	 * fill the table 'installerfunction'
	 * @throws SQLException
	 */
	private static void initializeDB_InstallerFunction() throws SQLException {		
		//for each valid groundfunction check if it is in db
		for(int i = 0; i < installerFunctionInfoHash.get(1).size(); i++) {
			if(existsInstallerFunctionInDB(installerFunctionInfoHash.get(1).get(i))) {
				continue;
			} else {
				query = "INSERT INTO installerfunction (functionname) VALUES (?)";
				prepState = con.prepareStatement(query);
				prepState.setString(1, installerFunctionInfoHash.get(1).get(i));
				result = prepState.executeQuery();			
			}			
		}		
	}
	
	/**
	 * check if patchHandler contains "API::Patch::"
	 * @param patchHandler | name of PatchHandler | String
	 * @return name of PatchHandler | String
	 */
	private static String getHandlerPackage(String patchHandler) {
		String basic = "API::Patch::";
		if(patchHandler.contains(basic)) {
			return patchHandler;
		} else {
			return basic + patchHandler;
		}
	}
	
	/**
	 * check if the installer function is already in the table 'installerfunction'
	 * @param functionName | name of installer function | String
	 * @return true if already exists, false otherwise | boolean
	 * @throws SQLException
	 */
	private static boolean existsInstallerFunctionInDB(String functionName) throws SQLException {
		if(!validInstallerFunction(functionName)) {
			System.out.println(String.format("%s is not a valid installer function", functionName));
			return false;
		}else {
			try {
				query = "SELECT * FROM installerfunction WHERE  functionname= ?";
				prepState = con.prepareStatement(query);
				prepState.setString(1, functionName);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());				
			}
			
			if(result.next()) {
				return true;
			} else {
				return false;
			}
		}
	}
	
	/**
	 * check if the patch ground function is already in the table 'patchgroundfunction'
	 * @param functionName | name of ground function | String
	 * @return true if already exists, false otherwise | boolean
	 * @throws SQLException
	 */
	private static boolean existsPatchGroundFunctionInDB(String functionName) throws SQLException {
		try {
			query = "SELECT * FROM patchgroundfunction WHERE functionname=?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, functionName);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			if(functionName.equals(result.getString(2))) {
				return true;
			} else {
				return false;
			}
		}else {
			return false;
		}
	}
	
	/**
	 * check if cartridge is already in the table 'patchcartridge'
	 * @param cartridgeName | name of cartridge | String
	 * @return true if already exists, false otherwise | boolean
	 * @throws SQLException
	 */
	private static boolean existsCartridgeInDB(String cartridgeName) throws SQLException {
		try {
			query = "SELECT * FROM patchcartridge WHERE name= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, cartridgeName);
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			if(cartridgeName.equals(result.getString(2))) {
				System.out.println(result.getString(2));
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * check if attribute name is locked and can not be set changed by setVersionAttribute
	 * @param attributeName | name of attribute | String
	 * @return true if attribute name is locked, false otherwise | boolean
	 */
	public static boolean lockedVersionKey(String attributeName) {
		
		Map<Integer, List<String>> validKeys = new HashMap<Integer, List<String>>();
		List<String> validFunctions = new ArrayList<String>();
		validFunctions.add("CartridgesVersionFolder");
		validFunctions.add("PatchCartridgesVersionFolder");
		validFunctions.add("PatchWebrootVersionFolder");
		validFunctions.add("CartridgesDataPackageFile");
		validFunctions.add("CartridgesVersionFolderFileList");
		validFunctions.add("CartridgesDataPackageUsedSourceURL");
		validFunctions.add("LanguagePackVersionFolderFileList");
		validKeys.put(1, validFunctions);
		
		if(validKeys.get(1).contains(attributeName)) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * return the current patch tag
	 * @return current patch tag | String
	 * @throws SQLException
	 */
	public static String getPatchTag() throws SQLException {
		String epagesTargetVersion = getBasicAttribute("ePagesTargetVersion");
		epagesTargetVersion = epagesTargetVersion.replace('.', '_');
		return "patch_" + epagesTargetVersion;
	}
	
	
	public static void initializePatchSystemInfoDB() {
		//TODO getcwd ?
	}
	
	public static void collectSystemInfo() throws UnknownHostException, SQLException {
		
		String OS = System.getProperty("os.name");
		String IPAddress = InetAddress.getLocalHost().getHostAddress();
		String DBS = "ROADWORKS";
		String OSVersion = "ROADWORKS";
		String DBSVersion = "ROADWORKS";
		String EpagesStartVersion; //TODO??
		String PerlExe; //TODO
		String PerlStartVersion = "ROADWORKS";
		String PerlTargetVersion = "ROADWORKS";
		String JavaStartVersion = "ROADWORKS";
		String JavaTargetVersion = "ROADWORKS";
		ArrayList<String> versions = validVersions();
		String EpagesCartridgesDir; //TODO
		String EpagesCartridgesDeEpages; //TODO
		String EpagesCartridgesVersionDir; //TODO= EpagesCartridgesDir + "/"; 
		String hostName; //TODO
		String EpagesDir; //TODO
	}
	
	/**
	 * 
	 * return a list with all existing languages
	 * @return list of all languages | ArrayList<String>
	 * @throws SQLException
	 */
	public static ArrayList<String> getAllLanguages() throws SQLException {
		try {
			stmt = con.createStatement();
			query = "SELECT label FROM patchlanguage ORDER BY label";
			result = stmt.executeQuery(query);
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		ArrayList<String> rs = new ArrayList<String>(); 
		while(result.next()) {
			rs.add(result.getString(1));
		}
		return rs;
	}
	
		
	/**
	 * add language to table 'patchlanguage'
	 * @param label | label of language | String
	 * @throws SQLException 
	 */
	public static void addLanguage(String label) throws SQLException {
		//check language
		if(existsLanguage(label)) {
			try {
				query = "INSERT INTO patchlanguage (label) VALUES (?)";
				prepState = con.prepareStatement(query);
				prepState.setString(1, label);
				prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());
			}
		}else {
			System.out.println(String.format("unable to register language, %s already exists", label));
			throw new SQLException();
		}
	}

	/**
	 * return the id of language label
	 * @param label | label of language | String
	 * @return language id | integer
	 * @throws SQLException
	 */
	private static int getLanguageID(String label) throws SQLException {
		try {
			//execute query
			query = "SELECT patchlanguageid FROM patchlanguage WHERE label= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, label);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return 1 if language exists
	 * @param label | label of language | String
	 * @return true if language exists, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean existsLanguage(String label) throws SQLException {
		try {
			query = "SELECT * FROM patchlanguage WHERE label= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, label);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			if(result.getString(1) == label) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * return the value of basic attribute
	 * @param name | name of the key | String
	 * @return value of basic attribute | String
	 * @throws SQLException
	 */
 	public static String getBasicAttribute(String name) throws SQLException {
		try {
			query = "SELECT value FROM patchsystemattribute WHERE keyname = ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, name);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get basic attribute for key, %s is not valid", name));
		    System.out.println(String.format("PatchSystemInfo ERROR no entry for basic attribute %s", name));
		}
		
		if(result.next()) {
			return result.getString(1);
		}else {
			return null;
		}
	}
 	
 	/**
 	 * check if the basic attribute exists
 	 * @param functionName | name of ground function | String
 	 * @return true if exists, false otherwise | boolean
 	 * @throws Exception 
 	 */
 	public static boolean existsBasicAttribute(String functionName) throws SQLException {
 		if(virtualBasicAttribute(functionName)) {
 			return true;
 		} else if(!(validBasicAttribute(functionName))) {
 			System.out.println(String.format("unable to check system info for key, "
 					+ "'%s' is not valid", functionName));
 			return false;
 		} else {
 			try {
 				query = "SELECT * FROM patchsystemattribute WHERE keyname= ?";
 				prepState = con.prepareStatement(query);
 				prepState.setString(1, functionName);
 				result = prepState.executeQuery();
 			} catch(SQLException e) {
 				//handle errors
 			    System.out.println("SQLException: " + e.getMessage());
 			    System.out.println("SQLState: " + e.getSQLState());
 			    System.out.println("VendorError: " + e.getErrorCode());
 			}
 			if(result.next()) {
 				if(functionName.equals(result.getString(2)));
 				return true;
 			} else {
 				return false;
 			}
 		}
 	}
 	
 	/**
	 * check if attribute name is valid
	 * @param attributeName | name of attribute | String
	 * @return true if attribute is valid, false otherwise | boolean
	 */
	public static boolean validBasicAttribute(String attributeName) {
		
		Map<Integer, List<String>> validKeys = new HashMap<Integer, List<String>>(); //key, list of valid values
		List<String> validValues = new ArrayList<String>(); //list of valid values
		//epages directory values
		
		validValues.add("EpagesDir");
		validValues.add("EpagesBinDir");
		validValues.add("EpagesSharedDir");
		validValues.add("EpagesPerlDir");
		validValues.add("EpagesCartridgesDir");
		validValues.add("EpagesCartridgesDeEpages");
		validValues.add("EpagesCartridgesVersionDir");
		validValues.add("EpagesConfigDir");
		validValues.add("EpagesConfigDirStoreTypes");
		validValues.add("EpagesWebrootDir");
		validValues.add("EpagesConfigVersionFile");
		validValues.add("EpagesDatabaseConfigDir");
		validValues.add("EpagesStatic");
		validValues.add("EpagesSharedPatchesDir");
		validValues.add("EpagesLanguagePackDir");
		validValues.add("EpagesDataPackageDir");
		
		//patch dir values
		validValues.add("PatchDir");
		validValues.add("PatchDataDir");
		validValues.add("PatchModuleTemplatesDir");
		validValues.add("PatchConfigDirFeatureRefs");
		validValues.add("PatchConfigDirFlexFeatureRefs");
		validValues.add("PatchConfigDirCartridges");
		validValues.add("PatchConfigDirCartridgesXML");
		validValues.add("PatchConfigDirConfigurationFiles");
		validValues.add("PatchTargetDir");
		validValues.add("PatchLogDir");
		validValues.add("PatchCartridgesVersionDir");
		validValues.add("PatchConfigDir");
		validValues.add("PatchWebrootVersionDir");
		validValues.add("PatchCartridgesDataPackageExclusiveURLs");
		
		//backup dir values
		validValues.add("PatchBackupDir");
		validValues.add("WebrootBackUpDirectory");
		
		//info values
		validValues.add("OperatingSystem");
		validValues.add("IPAddress");
		validValues.add("DataBaseSystem");
		validValues.add("OperatingSystemVersion");
		validValues.add("DataBaseSystemVersion");
		validValues.add("ePagesStartVersion");
		validValues.add("ePagesTargetVersion");
		validValues.add("PerlStartVersion");
		validValues.add("PerlTargetVersion");
		validValues.add("PerlExe");
		validValues.add("JavaStartVersion");
		validValues.add("JavaTargetVersion");
		validValues.add("PatchCallHandlerScript");
		validValues.add("PatchDisplayReportScript");
		validValues.add("DisplayHTMLReportProcessID");
		validValues.add("HTMLReportTargetFolder");
		validValues.add("EpagesConfFile");
		validValues.add("DataCacheConfFile");
		validValues.add("EpagesSchedulerConfFile");
		validValues.add("EpagesWebInterfaceConfFile");
		validValues.add("PatchStatisticsDir");
		validValues.add("EpagesServletConfFile");
		
		//configuration values
		validValues.add("PatchCartridgesXML");
		validValues.add("PatchScheduler");
		validValues.add("PatchWebInterface");
		validValues.add("PatchEpagesConf");
		validValues.add("PatchDataCacheConf");
		validValues.add("PatchServletConf");
		validValues.add("RemoveUnusedVersionFolders");
		validValues.add("BackupWebrootFolder");
		validValues.add("BuildUIOnlyFinalVersion");
		validValues.add("BuildDojoOnlyFinalVersion");
		validValues.add("BuildStoreTypesOnlyFinalVersion");
		validValues.add("BuildJQueryOnlyFinalVersion");
		validValues.add("MinifyOnlyFinalVersion");
		validValues.add("AutoCleanUpPatchFiles");
		validValues.add("PatchStoresParallel");
		validValues.add("PatchPrepareParallel");
		validValues.add("PatchStoresParallelSimultaneous");
		validValues.add("PatchStoresParallelTriesAfterError");
		validValues.add("PatchStoresParallelWaitTime");
		validValues.add("PatchStoresParallelWaitTimeAfterError");
		validValues.add("DisplayIntervallHTMLReport");
		validValues.add("WriteHTMLReport");
		validValues.add("ExecuteBrowserToDisplayHTMLReport");
		validValues.add("DisplayStatisticReport");
		validValues.add("AutoRecreateSolrSearchIndex");
		validValues.add("AutoSynchronizeStores");
		validValues.add("TimeMeasurement");
		validValues.add("SkipNonStandardVersionCartridges");
		validValues.add("AutoSynchronizeSite");
		
		//patch system info
		validValues.add("PatchSystemInfoGlobalOverwrite");
		
		//control parameter
		validValues.add("NextStoreToPatch");
		
		//add Group with key=1 to validkeys
		validKeys.put(1, validValues);
		
		if(validValues.contains(attributeName) && validKeys.containsValue(validValues)) {
			return true;
		} else {
			return false;
		}
	}

	/**
 	 * check if attribute name is a virtual attribute
 	 * @param attributeName | name of attribute | String
 	 * @return true if attribute name is virtual, 0 otherwise | boolean
 	 */
 	public static boolean virtualBasicAttribute(String attributeName) {
 		
 		Map<Integer, String> virtualKeys = new HashMap<Integer, String>();
 		virtualKeys.put(1, "ePagesCurrentVersion");
 		
 		if(virtualKeys.containsValue(attributeName)) {
 			return true;
 		} else {
 			return false;
 		}
 	}
 		
	/**
	 * Returns Label by ID
	 * @param: id | version id | integer
	 * @return: label of a version | String
	 * @throws SQLException
	 */
	private static String getVersionLabel(int id) throws SQLException {	
		//Get query 
		try {
			query = ("SELECT label FROM patchversion WHERE patchversionid = ?");
			prepState = con.prepareStatement(query);
			prepState.setInt(1, id);
			result = prepState.executeQuery();
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("PatchSystemInfoHandlerError: no id "
		    		+ "found in table 'patchversion' for id %d", id));
		    
		}
		
		if(result.next()) {
			return result.getString(1);
		} else {
			return null;
		}
	}
	
	
	/**
	 * returns the id of a version
	 * @param label | label of epages version | String
	 * @return version id | integer
	 * @throws SQLException
	 */
	private static int getVersionID(String label) throws SQLException {	
		if(!validVersion(label)) {
			System.out.println(String.format("unable to get version id, %s is not valid", label));
		}
		
		//Get query 
		try {
			query = ("SELECT patchversionid FROM patchversion WHERE label = ?");
			prepState = con.prepareStatement(query);
			prepState.setString(1, label);
			result = prepState.executeQuery();
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		} 
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	
	/**
	 * returns position of version
	 * @param label | label of epages release version | String
	 * @return position of a version | integer
	 * @throws SQLException
	 */
	private static int getVersionOrder(String label) throws SQLException {	
		//Get query 
		try {
			query = ("SELECT orderid FROM patchversion WHERE label = ?");
			prepState = con.prepareStatement(query);
			prepState.setString(1, label);
			result = prepState.executeQuery();
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get version id, %s is not valid", label));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return the value of version attribute
	 * @param label | label of epages release version | String
	 * @param keyLabel | key of patch version attribute | String
	 * @return value of version attribute | String
	 * @throws SQLException
	 */
	public static String getVersionAttribute(String label, String keyLabel) throws SQLException {
		int versionID = 0;
		int versionKeyID = 0;
		
		try {
			versionID = getVersionID(label);
			versionKeyID = getPatchVersionKeyID(keyLabel);

			try {
				query = "SELECT value FROM patchversionattribute WHERE patchversionid=? and "
						+ "patchversionkeyid= ?";
				prepState = con.prepareStatement(query);
				prepState.setInt(1, versionID);
				prepState.setInt(2, versionKeyID);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());
			}
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			return result.getString(1);
		} else {
			return null;
		}
	}
	
	/**
	 * Update version label in the database
	 * @param id | patchversion id in the database | integer
	 * @param label | label of a version | String
	 */
	static void setVersionLabel(int id, String label) {
		try {
			query = ("UPDATE patchversion SET label = ? WHERE patchversionid = ?");
			prepState = con.prepareStatement(query);
			
			//Counter variables
			int i = 1;
			prepState.setString(i, label);
			prepState.setInt(i++, id);
			result = stmt.executeQuery(query);
			System.out.println("version label updated successfully");
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("version with id %d does not exist", id));
		}
	}
	
	/**
	 * check if version is the last version in the version path
	 * @param label | label of epages release version | String
	 * @return true if version is last version, 0 otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean isLastVersion(String label) throws SQLException {
		if(!validVersion(label)) {
			System.out.println("%s is not a valid version");
			return false;
		} else if(label.equals(validVersions().get(validVersions().size() - 1))){
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * check if version is the first version in the version path
	 * @param label | label of epages release version | String
	 * @return true if version is the first version, false otherwise | boolean 
	 * @throws SQLException
	 */
	public static boolean isFirstVersion(String label) throws SQLException {
		if(!validVersion(label)) {
			System.out.println("%s is not a valid version");
			return false;
		} else if(label.equals(validVersions().get(0))) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * return the id of a patch handler
	 * @param pack | patch handler package | String
	 * @return patchhandlerid | handler id in database | integer
	 * @throws SQLException
	 */
	private static int getPatchHandlerID(String pack) throws SQLException {
		try {
			query = ("SELECT patchhandlerid FROM patchhandler WHERE package = ?");
			prepState = con.prepareStatement(query);
			
			prepState.setString(1, pack);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get handler id, %s does not exist", pack));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return id of patch ground function
	 * @param fname | ground function name | String
	 * @return ground function id | integer
	 * @throws SQLException
	 */
	private static int getPatchGroundFunctionID(String fname) throws SQLException {
		try {
			query = ("SELECT patchgroundfunctionid FROM patchgroundfunction WHERE functionname = ?");
			prepState = con.prepareStatement(query);
			
			prepState.setString(1, fname);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get ground function id, %s is not valid", fname));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return the id of key, the key belongs to an version attribute
	 * @param kname | key of a version attribute | String 
	 * @return patch version key id in database | integer
	 * @throws SQLException
	 */
	private static int getPatchVersionKeyID(String kname) throws SQLException {
		try { 
			query = ("SELECT patchversionkeyid FROM patchversionkey WHERE keyname = ?");
			prepState = con.prepareStatement(query);
			
			prepState.setString(1, kname);
			result = prepState.executeQuery();
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get patch-version key id, %s is not in db", kname));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return the id of function
	 * @param fname name of installer function | String
	 * @return patch version key id in database | integer
	 * @throws SQLException
	 */
	private static int getInstallerFunctionID(String fname) throws SQLException {
		try {
			query = ("SELECT installerfunctionid from installerfunction WHERE functionname = ?");
			prepState = con.prepareStatement(query);
			
			prepState.setString(1, fname);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get installer function id, %s is not valid", fname));	
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return the id of store
	 * @param storeName | name of store | String
	 * @return store id | integer
	 * @throws SQLException
	 */
	private static int getStoreID(String storeName) throws SQLException {
		try {
			query = "SELECT patchstoreid FROM patchstore WHERE name = ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, storeName);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get store id, %s is not in database", storeName));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * return the id of cartridge
	 * @param storeName | name of store type | String
	 * @return cartridge id | integer
	 * @throws SQLException
	 */
	private static int getCartridgeID(String storeName) throws SQLException {
		try {
			query ="SELECT patchcartridgeid FROM patchcartridge WHERE name = ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, storeName);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get cartridge id, %s is not in database", storeName));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * check if cartridge installer function is completed for version, on store
	 * @param label | label of epages release version | String
	 * @param storeName | name of store | String
	 * @param cartridgeName | name of cartridge | String
	 * @param functionName | name of cartridge installer function | String
	 * @return true if function is completed, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean isCartridgeFunctionCompleted(String label, String storeName, String cartridgeName, String functionName ) throws SQLException {
		
		int versionID = 0;
		int storeID = 0;
		int cartridgeID = 0;
		int installerFunctionID = 0;
		
		try {
			//get id's
			versionID = getVersionID(label);
			storeID = getStoreID(storeName);
			cartridgeID = getCartridgeID(cartridgeName);
			installerFunctionID = getInstallerFunctionID(functionName);	
			
			try {
				//execute Statement
				query = "SELECT * FROM completedpatchstorecartridgeinstallerfunction WHERE "
						+ "patchstoreid = ? AND patchcartridgeid = ? AND patchversionid = ? "
						+ "AND installerfunctionid = ?";
				prepState = con.prepareStatement(query);
						
				int i = 1;
				prepState.setInt(i, storeID);
				prepState.setInt(i++, cartridgeID);
				prepState.setInt(i++, versionID);
				prepState.setInt(i++, installerFunctionID);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());				
			}
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		//check for content
		if(result.next()) {
			if(result.getInt(1) == storeID && result.getInt(2) == cartridgeID && 
					result.getInt(3) == versionID && result.getInt(4) == installerFunctionID) {
				return true;
			}else {
				return false;
			}			
		} else {
			return false;
		}
				
	}
	
	/**
	 * check if cartridge installer function is completed for version, on store
	 * @param label | label of epages release version | String
	 * @param storeName | name of store | String
	 * @param cartridgeName | name of cartridge | String
	 * @param functionName | name of cartridge installer function | String
	 * @return true if function is completed for version, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean isCartridgeFunctionStarted(String label, String storeName, String cartridgeName, String functionName) throws SQLException {
		int versionID = 0;
		int storeID = 0;
		int cartridgeID = 0;
		int installerFunctionID = 0;
		
		try {
			//get id's
			versionID = getVersionID(label);
			storeID = getStoreID(storeName);
			cartridgeID = getCartridgeID(cartridgeName);
			installerFunctionID = getInstallerFunctionID(functionName);	
			
			try {
				//execute Statement
				query = "SELECT * FROM startedpatchstorecartridgeinstallerfunction WHERE "
						+ "patchstoreid = ? AND patchcartridgeid = ? AND patchversionid = ? "
						+ "AND installerfunctionid = ?";
				prepState = con.prepareStatement(query);
						
				int i = 1;
				prepState.setInt(i, storeID);
				prepState.setInt(i++, cartridgeID);
				prepState.setInt(i++, versionID);
				prepState.setInt(i++, installerFunctionID);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());				
			}
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		//check for content
		if(result.next()) {
			if(result.getInt(1) == storeID && result.getInt(2) == cartridgeID && result.getInt(3) == versionID && result.getInt(4) == installerFunctionID) {
				return true;
			}else {
				return false;
			}			
		} else {
			return false;
		}
	}
	
	/**
	 * return the id of patch host
	 * @param hostname | String
	 * @return host id in patchsysteminfodb | integer
	 * @throws SQLException
	 */
	private static int getPatchHostID (String hostname) throws SQLException {
		try {
			query = "SELECT patchhostid FROM patchhost WHERE patchhostname = ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, hostname);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to get host id, %s is not in database 'PatchSystemInfoDB'", hostname));
		}
		
		if(result.next()) {
			return result.getInt(1);
		} else {
			return -1;
		}
	}
	
	/**
	 * check if store cartridge is started for version	
	 * @param label | label if epages release version | String
	 * @param storeName | name of store | String
	 * @param cartridgeName | name of cartridge | String
	 * @return true if function is started for version, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean isStoreCartridgeStarted(String label, String storeName, String cartridgeName) throws SQLException {
		int versionID = 0;
		int storeID = 0;
		int cartridgeID = 0;
		
		try {
			//get id's
			versionID = getVersionID(label);
			storeID = getStoreID(storeName);
			cartridgeID = getCartridgeID(cartridgeName);
			
			try {
				//execute statement
				query = "SELECT * from startedpatchstorecartridge WHERE patchstoreid=? AND "
						+ "patchcartridgeid= ? AND patchversionid= ?";
				prepState = con.prepareStatement(query);
				
				int i = 1;
				prepState.setInt(i, storeID);
				prepState.setInt(i++, cartridgeID);
				prepState.setInt(i++, versionID);
				
				result = prepState.executeQuery();			
			}catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());
			}
		}catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		//check for content
		if(result.next()) {
			if(result.getInt(1) == storeID && result.getInt(2) == cartridgeID && result.getInt(3) == versionID) {
				return true;
				}else {
					return false;
				}			
			} else {
				return false;
			}
	}
		
	/**
	 * Add new entry to the database
	 * @param: orderid | Orderid | integer
	 * @param: label | Label | String
	 */
	static void add(int orderid, String label) {
		try {
			query = ("INSERT INTO patchversion (orderid, label) VALUES (?, ?)");
			prepState = con.prepareStatement(query);
			
			int i = 1;
			prepState.setInt(i, orderid);
			prepState.setString(i++, label);
			prepState.executeQuery();
			System.out.println("patchversion added successfully");
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		} finally {
			//TODO update entry instead of insert?
			//set(id, label);
		}
	}
	
	/**
	 * deletes row in database
	 * @param: id | ID | integer
	 */
	static void delete(int id) {
		try {
			query = "DELETE FROM patchversion WHERE patchversionid = ?";
			prepState = con.prepareStatement(query);
			prepState.setInt(1, id);
			prepState.execute(query);
			System.out.println("entry deleted successfully");
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		    System.out.println(String.format("unable to delete entry, %d is not valid", id));
		}
	}
	
	/**
	 * set flag for use of standard versions, 1 cartridge use standard versions, 
	 * 0 cartridge not use standard versions
	 * @param cartridgeName | cartridge name | String
	 * @param stdVersion | true cartridge use standard versions, false cartridge not use standard versions
	 * @throws SQLException
	 */
	public static void setCartridgeUseStandardVersions(String cartridgeName, boolean stdVersion) throws SQLException {
		
		if(stdVersion != false || stdVersion != true) {
			System.out.println(String.format("no boolean attribute %b", stdVersion));
			throw new SQLException(); 
		}
		
		if(!existsCartridgeInDB(cartridgeName)) {
			try {
				query = "INSERT INTO patchcartridge (name, usestandardversions) VALUES (?, ?)";
				prepState = con.prepareStatement(query);
				
				int i = 1;
				prepState.setString(i, cartridgeName);
				prepState.setBoolean(i++, stdVersion);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());
			}
		} else {
			try {
				query = "UPDATE patchcartidge SET usestandardversions= ? WHERE name=?";
				prepState = con.prepareStatement(query);
				
				int i = 1;
				prepState.setBoolean(i, stdVersion);
				prepState.setString(i++, cartridgeName);
				result = prepState.executeQuery();
			} catch(SQLException e) {
				//handle errors
			    System.out.println("SQLException: " + e.getMessage());
			    System.out.println("SQLState: " + e.getSQLState());
			    System.out.println("VendorError: " + e.getErrorCode());
			}
		}
	}
	
	/**
	 * check if version is valid
	 * @param label | label of version | String
	 * @return true if attribute name is valid, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean validVersion(String label) throws SQLException {
		try {
			query = "SELECT * FROM patchversion WHERE label= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, label);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		//TODO check/set cache
		if(result.next()) {
			return true;
		}else {
			return false;
		}
	}
	
	/**
	 * get a list of all valid versions
	 * @return list of all valid versions | ArrayList<String>
	 * @throws SQLException
	 */
	public static ArrayList<String> validVersions() throws SQLException {
		try {
			stmt = con.createStatement();
			query = "SELECT label from patchversion ORDER BY orderid";
			result = stmt.executeQuery(query);
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		ArrayList<String> rs = new ArrayList<String>(); 
		while(result.next()) {
			rs.add(result.getString(1));
		}
		return rs;		
	}
	
	/**
	 * check if attribute name is valid
	 * @param pack | patch handler package | String
	 * @return true if patch handler package is valid, false otherwise | boolean
	 * @throws SQLException 
	 */
	public static boolean validHandler(String pack) throws SQLException {
		try {
			query = "SELECT patchhandlerid FROM patchhandler WHERE package = ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, pack);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		if(result.next()) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * check if installer function is valid
	 * @param functionName | function name | String
	 * @return true if function is valid, false otherwise | boolean
	 */
	public static boolean validInstallerFunction(String functionName) {
		if(installerFunctionInfoHash.get(1).contains(functionName)) {
			return true;
		} else {
			return false;
		} 
	}
	
	/**
	 * check if patch event exists
	 * @param eventName | name of patch event | String
	 * @return true if already exists, false otherwise | boolean
	 * @throws SQLException
	 */
	public static boolean existsPatchEvent(String eventName) throws SQLException {
		try {
			query = "SELECT * FROM patchevent WHERE name= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, eventName);
			result = prepState.executeQuery();
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			if(eventName.equals(result.getString(2))) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * set current time for a patch event
	 * @param eventName | name of patch event | String
	 * @throws SQLException
	 */
	public static void registerPatchEventTime(String eventName) throws SQLException {
		
		if(!existsPatchEvent(eventName)) {
			query = "INSERT INTO patchevent (name, pointoftime) VALUES (?, NOW())";
			prepState = con.prepareStatement(query);
			prepState.setString(1, eventName);
			prepState.executeQuery();			

		} else {
			query = "UPDATE patchevent SET pointoftime = NOW() WHERE name= ?";
			prepState = con.prepareStatement(query);
			prepState.setString(1, eventName);
			prepState.executeQuery();
		} 
	}
	
	/**
	 * set patch start time
	 * @throws SQLException
	 */
	public static void setStartTime() throws SQLException {
		registerPatchEventTime("PatchStartTime");		
	}
	
	/**
	 * get patch start time
	 * @return time for event | time
	 * @throws SQLException
	 */
	public static Time getStartTime() throws SQLException {
		try {
			stmt = con.createStatement();
			query = "SELECT NOW()";
			result = stmt.executeQuery(query);
		} catch (SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}	
		
		if(result.next()) {
			return result.getTime(1);
		}else {
			return null;
		}
	}
		
	/**
	 * Get current time
	 * @return: current time | String
	 * @throws SQLException
	 */
	public static Time getCurrentTime() throws SQLException {
		try {
			stmt = con.createStatement();
			query = "SELECT NOW()";
			result = stmt.executeQuery(query);
		} catch(SQLException e) {
			//handle errors
		    System.out.println("SQLException: " + e.getMessage());
		    System.out.println("SQLState: " + e.getSQLState());
		    System.out.println("VendorError: " + e.getErrorCode());
		}
		
		if(result.next()) {
			return result.getTime(1);
		} else {
			return null;
		}		
	}
}

